@@ -1,7 +1,8 @@
-# coding: utf-8
+
+from __future__ import annotations
 
 import datetime
-import base64
+from datetime import timedelta as TimeDelta
 import binascii
 import sys
 import types
@@ -17,6 +18,7 @@ from ruamel.yaml.compat import (builtins_module, # NOQA
                                 nprint, nprintf, version_tnf)
 from ruamel.yaml.compat import ordereddict
 
+from ruamel.yaml.tag import Tag
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
                                   CommentedKeySeq, CommentedSeq, TaggedScalar,
@@ -33,7 +35,8 @@ from ruamel.yaml.scalarbool import ScalarBoolean
 from ruamel.yaml.timestamp import TimeStamp
 from ruamel.yaml.util import timestamp_regexp, create_timestamp
 
-from typing import Any, Dict, List, Set, Iterator, Union, Optional  # NOQA
+if False:  # MYPY
+    from typing import Any, Dict, List, Set, Iterator, Union, Optional  # NOQA
 
 
 __all__ = ['BaseConstructor', 'SafeConstructor', 'Constructor',
@@ -310,11 +313,17 @@ class BaseConstructor:
             pairs.append((key, value))
         return pairs
 
+    # ToDo: putting stuff on the class makes it global, consider making this to work on an
+    # instance variable once function load is dropped.
     @classmethod
-    def add_constructor(cls, tag: Any, constructor: Any) -> None:
+    def add_constructor(cls, tag: Any, constructor: Any) -> Any:
+        if isinstance(tag, Tag):
+            tag = str(tag)
         if 'yaml_constructors' not in cls.__dict__:
             cls.yaml_constructors = cls.yaml_constructors.copy()
+        ret_val = cls.yaml_constructors.get(tag, None)
         cls.yaml_constructors[tag] = constructor
+        return ret_val
 
     @classmethod
     def add_multi_constructor(cls, tag_prefix: Any, multi_constructor: Any) -> None:
@@ -504,6 +513,8 @@ class SafeConstructor(BaseConstructor):
             return sign * float(value_s)
 
     def construct_yaml_binary(self, node: Any) -> Any:
+        import base64
+
         try:
             value = self.construct_scalar(node).encode('ascii')
         except UnicodeEncodeError as exc:
@@ -655,6 +666,8 @@ class Constructor(SafeConstructor):
         return self.construct_scalar(node)
 
     def construct_python_bytes(self, node: Any) -> Any:
+        import base64
+
         try:
             value = self.construct_scalar(node).encode('ascii')
         except UnicodeEncodeError as exc:
@@ -1006,13 +1019,11 @@ class RoundTripConstructor(SafeConstructor):
             underscore = None
         value_s = value_su.replace('_', "")
         sign = +1
-        if value_s[0] == '-':
-            sign = -1
         if value_s[0] in '+-':
+            if value_s[0] == '-':
+                sign = -1
             value_s = value_s[1:]
-        if value_s == '0':
-            return 0
-        elif value_s.startswith('0b'):
+        if value_s.startswith('0b'):
             if self.resolver.processing_version > (1, 1) and value_s[2] == '0':
                 width = len(value_s[2:])
             if underscore is not None:
@@ -1074,7 +1085,12 @@ class RoundTripConstructor(SafeConstructor):
             if underscore is not None:
                 # cannot have a leading underscore
                 underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
-            return ScalarInt(sign * int(value_s), width=len(value_s), underscore=underscore)
+            return ScalarInt(
+                sign * int(value_s),
+                width=len(value_s),
+                underscore=underscore,
+                anchor=node.anchor,
+            )
         elif underscore:
             # cannot have a leading underscore
             underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
@@ -1210,8 +1226,7 @@ class RoundTripConstructor(SafeConstructor):
     def flatten_mapping(self, node: Any) -> Any:
         """
         This implements the merge key feature http://yaml.org/type/merge.html
-        by inserting keys from the merge dict/list of dicts if not yet
-        available in this node
+        by referencing the merge dict/list of dicts
         """
 
         def constructed(value_node: Any) -> Any:
@@ -1480,14 +1495,17 @@ class RoundTripConstructor(SafeConstructor):
             state = SafeConstructor.construct_mapping(self, node, deep=True)
             data.__setstate__(state)
         elif is_dataclass(data):
-            mapping = SafeConstructor.construct_mapping(self, node)
+            mapping = SafeConstructor.construct_mapping(self, node, deep=True)
             init_var_defaults = {}
             for field in data.__dataclass_fields__.values():
                 # nprintf('field', field, field.default is MISSING,
                 #          isinstance(field.type, InitVar))
                 # in 3.7, InitVar is a singleton
                 if (
-                    isinstance(field.type, InitVar) or field.type is InitVar
+                    isinstance(field.type, InitVar)
+                    or field.type is InitVar
+                    # this following is for handling from __future__ import allocations
+                    or (isinstance(field.type, str) and field.type.startswith('InitVar'))
                 ) and field.default is not MISSING:
                     init_var_defaults[field.name] = field.default
             for attr, value in mapping.items():
@@ -1581,6 +1599,10 @@ class RoundTripConstructor(SafeConstructor):
     def construct_yaml_set(self, node: Any) -> Iterator[CommentedSet]:
         data = CommentedSet()
         data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
+        if node.flow_style is True:
+            data.fa.set_flow_style()
+        elif node.flow_style is False:
+            data.fa.set_block_style()
         yield data
         self.construct_setting(node, data)
 
@@ -1665,20 +1687,21 @@ class RoundTripConstructor(SafeConstructor):
         else:
             return create_timestamp(**values)
             # return SafeConstructor.construct_yaml_timestamp(self, node, values)
-        dd = create_timestamp(**values)  # this has delta applied
+        # print('>>>>>>>> here', values)
+        dd = create_timestamp(**values)  # this has tzinfo
         delta = None
         if values['tz_sign']:
-            tz_hour = int(values['tz_hour'])
+            hours = values['tz_hour']
+            tz_hour = int(hours)
             minutes = values['tz_minute']
             tz_minute = int(minutes) if minutes else 0
-            delta = datetime.timedelta(hours=tz_hour, minutes=tz_minute)
+            # ToDo: double work, replace with extraction from dd.tzinfo
+            delta = TimeDelta(hours=tz_hour, minutes=tz_minute)
             if values['tz_sign'] == '-':
                 delta = -delta
-        # should check for None and solve issue 366 should be tzinfo=delta)
-        # isinstance(datetime.datetime.now, datetime.date) is true)
         if isinstance(dd, datetime.datetime):
             data = TimeStamp(
-                dd.year, dd.month, dd.day, dd.hour, dd.minute, dd.second, dd.microsecond,
+                dd.year, dd.month, dd.day, dd.hour, dd.minute, dd.second, dd.microsecond, dd.tzinfo,  # NOQA
             )
         else:
             # ToDo: make this into a DateStamp?
